//This code contains routines for the analog to digital conversion.
//Author:  John Seng

#include "globals.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/delay.h>

u08 volatile analog_data[11]; //the data is stored in here
u08 current_sensor=0;
u08 current_4051_input=0;

//return the value of an analog input
u08 analog(u08 num) {
  return analog_data[num+1];
}

//returns the value of the knob
u08 knob() {
  return analog_data[0];
}

void adc_init(void) {
  ADCSRA |= _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0) | _BV(ADIE); 
  //enable ADC and set prescalar to /128 
  //enable ADC interrupt

  ADMUX |= _BV(ADLAR); //left shift data 

  sei();

  ADCSRA |= _BV(ADSC);  //get first reading

  DDRA |= 0xE0; //set bits 5,6,7 to be outputs
}

INTERRUPT(SIG_ADC) {
  //get next value
  if (current_sensor == 0 && current_4051_input < 7) {
    analog_data[current_4051_input] = ADCH;  //read value
    current_4051_input++;
    cbi(PORTA,7);
    cbi(PORTA,6);
    cbi(PORTA,5);
    PORTA |= (current_4051_input << 5);
    _delay_loop_1(3); //delay for 4051
  } else if (current_sensor == 0 && current_4051_input == 7) {
    analog_data[current_4051_input] = ADCH;  //read value
    current_sensor++;
    cbi(PORTA,7);
    cbi(PORTA,6);
    cbi(PORTA,5);
  } else {
    analog_data[current_sensor+current_4051_input] = ADCH;  //read value
    current_sensor++;
    if (current_sensor > 3) {
      current_sensor=0;
      current_4051_input=0;
    }
  }

  ADMUX &= 0xE0; //clear the lower 5 bits

  ADMUX |= current_sensor;

  ADCSRA |= _BV(ADSC); //start next reading
}

